package orderprocessing;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.text.ParseException;

/**
*	OrderEntry is the GUI to this application for order entry. The application can
*    have other front ends for other purposes such as customer maintenance.
*	Generally it is well designed, with a good event structure. However, it does
*	incorporate some logic that should be elsewhere, as noted.
*	It also writes to System.out in places in which it should use dialog boxes instead.
* 	To do order entry, this should be your "main" class.
*/
public class OrderEntry extends Frame
{	public OrderEntry()
	{	super("OrderEntry");
    	setBounds(20, 20, 640, 480);
        addWindowListener
        (	new WindowAdapter() // Anonymous inner class
        	{	public void windowClosing(WindowEvent e)
        		{	OrderProcessor.close(OrderProcessor.databaseName()); // Do this when you want orders saved.
        			System.exit(0);
        		}
        	}
		);
        GridBagLayout layout = new GridBagLayout();
        setLayout(layout);
        add(new Label("Order Entry"), 1,0,1,1,0.0, 1.0);
        Button getCustomerInfo = new Button("Get Customer Information");
        CustomerListener cl = new CustomerListener();
        customerNumber.addActionListener(cl);
        getCustomerInfo.addActionListener(cl);
        customerName.setEnabled(false);
        customerAddress.setEnabled(false);
        add(new Label("Customer Number"), 0,1,1,1,0.0,0.0 );
        add(customerNumber, 1,1,1,1,0.0, 1.0);
        add(getCustomerInfo,2,1,1,1,0.0, 0.0);
        add(new Label("Customer Name"), 0, 3, 1,1,0.0,0.0);
        add(customerName,   1,3,4,1,0.0, 1.0);
        add(new Label("Billing Address"), 0, 4, 1,1,0.0,0.0);
        add(customerAddress,1,4,4,3,0.0, 1.0);
        add(new Label("Shipping Address"), 0, 8, 1,1,0.0,0.0);
        add(new Label("(Keep this line structure.)"), 0, 9, 1, 1, 0.0,0.0);
        add(shippingAddress,1,8,4,3,0.0, 1.0);

        add(new Label("Billing Terms"), 0,12,1,1,0.0,0.0 );
        billingTerms.add("Default");
        billingTerms.add("NET 10");
        billingTerms.add("NET 30");
        billingTerms.add("EOM 10");
        add(billingTerms,1,12,1,1,0.0,0.0);

        add(new Label("Order Date"), 0,13,1,1,0.0,0.0 );
        add(new Label("Ship Date"),  1,13,1,1,0.0,0.0 );
        add(new Label("Cancel Date"),2,13,1,1,0.0,0.0 );
        add(orderDateField, 0,14,1,1,0.0,0.0 );
        add(shipDateField,  1,14,1,1,0.0,0.0 );
        add(cancelDateField,2,14,1,1,0.0,0.0 );
        add(new Label("Item Number (numeric)"),0,16,1,1,0.0,0.0 );
        add(itemNumberField, 1,16,1,1,0.0,0.0 );
        add(new Label("Quantity"),0,17,1,1,0.0,0.0 );
        add(quantityField, 1,17,1,1,0.0,0.0 );
        add(new Label("Unit Price"),0,18,1,1,0.0,0.0 );
        add(priceField, 1,18,1,1,0.0,0.0 );
        add(new Label("Description"),0,19,1,1,0.0,0.0 );
        add(descriptionField, 1,19,1,1,0.0,0.0 );
        add(accept, 0,20,1,1,0.0,0.0 );
        add(enter, 2,20,1,1,0.0,0.0 );

        accept.addActionListener(new AcceptListener());
        enter.addActionListener(new EnterListener());
        setBackground(new Color(175, 175, 255));
    	setVisible(true);
        original = enter.getBackground();
	}

    /** Convenience method to aid adding a component to a grid bag layout
	 * @param arg The component to be added
	 * @param x the x location (column) in the layout for the upper edge
	 * @param y the row in the layout for the left edge
	 * @param h the number of cells to use horizontally
	 * @param v the number of cells to used vertically
	 * @param wx how to balance horizontally if not all cells are filled
	 * @param wy how to balance vertically if not all cells are filled
	 */
	private void add(Component arg, int x, int y, int h, int v, double wx, double wy)
    {	GridBagLayout gbl = (GridBagLayout)getLayout();
		GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.NONE;
        c.gridx = x;
        c.gridy = y;
        c.gridwidth = h;
        c.gridheight = v;
        c.weightx = wx;
        c.weighty = wy;
        gbl.setConstraints(arg, c);
        add(arg);
    }

   private TextField customerNumber = new TextField(10);
   private TextField customerName = new TextField(30);
   private TextArea customerAddress = new TextArea("",5, 40,TextArea.SCROLLBARS_NONE);
   private TextArea shippingAddress = new TextArea("",5, 40,TextArea.SCROLLBARS_NONE);
   private Choice billingTerms = new Choice();
   private TextField orderDateField = new TextField(25);
   private TextField shipDateField = new TextField(25);
   private TextField cancelDateField = new TextField(25);
   private TextField itemNumberField = new TextField(10);
   private TextField quantityField = new TextField(10);
   private TextField priceField = new TextField(10);
   private TextField descriptionField = new TextField(30);
   private Button accept = new Button("Accept Item");
   private Button enter = new Button("Enter Complete Order");
   private Color original = null;
   private ArrayList items = new ArrayList();
   private SimpleDateFormat sdf = new SimpleDateFormat("EEE MMM dd, yyyy");

   /**
 * Listener for the customer button and customer number text field
 *
 */
private class CustomerListener implements ActionListener
    {	public void actionPerformed(ActionEvent e)
    	{   String cn = customerNumber.getText().trim();
            if (cn.equals(""))
            {	customerName.setText("Enter a customer number above.");
        		customerNumber.requestFocus();
            	return;
            }
    		cn = cn.toUpperCase();
            customerNumber.setText(cn);
    		Customer c = Customer.get(cn);
            if (c == null)
            { 	customerName.setText("No such customer.");
        		customerAddress.setText("");
                shippingAddress.setText("");
                billingTerms.select("Default");
                customerNumber.selectAll();
                customerNumber.requestFocus();
            	return;
            }
            customerName.setText(c.name());
            customerAddress.setText("" + c.billingAddress() + "       " + c.billingTerms());
            shippingAddress.setText("" + c.shippingAddress());

            billingTerms.select(c.billingTerms().code()); // is equivalent to the above.

            CalculationCalendar od = new CalculationCalendar() ;
            orderDateField.setText(sdf.format(od.getTime()));
            CalculationCalendar sd = od.incrementedBy(Calendar.DAY_OF_MONTH ,7);
            CalculationCalendar cd = sd.incrementedBy(Calendar.DAY_OF_MONTH ,10);
            shipDateField.setText(sdf.format(sd.getTime()));
            cancelDateField.setText(sdf.format(cd.getTime()));
            itemNumberField.requestFocus();

            items.clear(); // make sure any earlier items are removed.
            enter.setBackground(original);
            enter.setLabel("Enter Complete Order");
    	}
    }

	/**
	 * Listener for the accept order item button
	 *
	 */
	private class AcceptListener implements ActionListener
	{	public void actionPerformed(ActionEvent evt)
		{	/* Note that only blank field editing has been done. It needs more.
        		The logic of the editing should be put with the appropriate objects
                        in an "edit" method and not here.
                        */
        	String 	item = "";
            String temp = itemNumberField.getText().trim();
			if (temp.equals(""))
            {	itemNumberField.setText("Illegal");
				itemNumberField.selectAll();
				itemNumberField.requestFocus();
				return;
            }
        	item += temp+ "~";
            temp = quantityField.getText().trim();
       		if (temp.equals(""))
        	{	quantityField.setText("Illegal");
       			quantityField.selectAll();
       			quantityField.requestFocus();
       			return;
        	}
        	item += temp + "~";
        	temp = priceField.getText().trim();
        	if (temp.equals(""))
        	{	priceField.setText("Illegal");
        		priceField.selectAll();
        		priceField.requestFocus();
        		return;
        	}
       	 	item += temp + "~";
        	temp = descriptionField.getText().trim();
        	if (temp.equals(""))
        	{	descriptionField.setText("Illegal");
        		descriptionField.selectAll();
        		descriptionField.requestFocus();
        		return;
        	}
        	item += temp;
                //System.out.println(item); // debug output
            items.add(item);
            itemNumberField.setText("");
            quantityField.setText("");
            priceField.setText("");
            descriptionField.setText("");
            item = "";
			itemNumberField.requestFocus();
        	enter.setBackground(Color.yellow);
        	enter.setLabel("Save Unsaved Order");
        }
	}

	/**
	 * Listener for the enter order button
	 *
	 */
   private class EnterListener implements ActionListener
    {	public void actionPerformed(ActionEvent evt)
    	{	CalculationCalendar od = new CalculationCalendar();
    		CalculationCalendar sd = new CalculationCalendar();
    		CalculationCalendar cd = new CalculationCalendar();
    		try
    		{	od.setTime(sdf.parse(orderDateField.getText()));
            	sd.setTime(sdf.parse(shipDateField.getText()));
            	cd.setTime(sdf.parse(cancelDateField.getText()));
    		}
            catch(ParseException e)
    		{	System.out.println("Illegal date");  // dialog box?
        		return;
    		}

            if (items.size() == 0)
            {   System.out.println("There are no items ordered."); // dialog box?
				return;
            }

            String bt = billingTerms.getSelectedItem();

            // Encode the ship address as required by its parser.
            // We should at least count the fields.
            String shipAddress = shippingAddress.getText().replace('\n','~');
            // System.out.println(shipAddress); // debug output

        	// System.out.println(items); // debug output

            String cust = customerNumber.getText().trim();

            try
            {	// create the order object.
				Order o = OrderProcessor.orderProcessor.newOrder
				(	cust,
					bt,
            		shipAddress,
                	items,
                	od, sd, cd
				);
				if (o == null)// if customer is illegal
				{ 	System.out.println("Failed order--No such customer."); // dialogs?
            		return;
				}
                // Not saving in database. Verify creation:
				o.display();
				// Testing only:
				OrderProcessor.orderProcessor.generateInvoice(o).display();
            }
            catch(Exception ex)
            {	System.out.println("Order creation failed.  " + ex);// dialogs?
            }
            items.clear();
            customerNumber.selectAll();
            customerNumber.requestFocus();
            enter.setBackground(original);
            enter.setLabel("Enter Complete Order");
		}
    }

	/** Create and display the order entry gui. Opens the database.
	 * @param args ignored
	 */
	public static void main(final String [] args)
	{   OrderProcessor.open(OrderProcessor.databaseName());
		new OrderEntry();
    }

}
